/* 
   TRURLib
   Copyright (C) 1999 Pawel A. Gajda (mis@k2.net.pl)

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */

/*
   $Id: nmalloc.c,v 1.7 2003/01/14 18:22:06 mis Exp $
 */


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "nassert.h"

typedef void (*memh) (void);

static memh mem_fail_fn = NULL;

void (*setxmallocs_handler(void (*handler) (void))) (void) {
    memh tmp = mem_fail_fn;
    mem_fail_fn = handler;
    return tmp;
}

void (*n_malloc_set_failhook(void (*fn) (void))) (void) {
    memh tmp = mem_fail_fn;
    mem_fail_fn = fn;
    return tmp;
}


static void nomem(void)
{
    if (mem_fail_fn != NULL)
        mem_fail_fn();
    else {
        printf("Memory exhausted\n");
        exit(1);
    }
}


void *n_malloc(size_t size)
{
    register void *v;

    if ((v = malloc(size)) == NULL)
	nomem();
    return v;
}

void *n_calloc(size_t nmemb, size_t size)
{
    register void *v = calloc(nmemb, size);
    if (v == 0)
	nomem();
    return v;
}

void *n_realloc(void *ptr, size_t size)
{
    n_assert(size > 0);
    if ((ptr = realloc(ptr, size)) == NULL)
	nomem();

    return ptr;
}


char *n_strdupl(const char *s, size_t length)
{
    register char *new;

    if ((new = n_malloc(length + 1)) == NULL) {
        nomem();
        return NULL;
    }

    memcpy(new, s, length);
    new[length] = '\0';
    return new;
}


char *n_strdup(const char *s)
{
    register size_t len = strlen(s) + 1;
    register char *new;

    if ((new = n_malloc(len)) == NULL) {
        nomem();
        return NULL;
    }

    return memcpy(new, s, len);
}


void *n_memdup(const void *ptr, size_t size)
{
    register void *new;

    if ((new = n_malloc(size)) == NULL) {
        nomem();
        return NULL;
    }

    return memcpy(new, ptr, size);
}


void n_free(void *ptr)
{
    if (ptr)
        free(ptr);
}

void n_cfree(void *ptr) 
{
    void **pptr = (void**)ptr;
    
    if (*pptr) {
        free(*pptr);
        *pptr = NULL;
    }
}

    
